# Copyright 2002-2008 Josh Clark and Global Moxie, LLC. This code cannot be
# redistributed without permission from globalmoxie.com.  For more
# information, consult your Big Medium license.
#
# $Id: PT.pm 3043 2008-03-31 14:00:38Z josh $

package BigMed::Search::Stopwords::PT;
use strict;
use warnings;
use utf8;

my @STOPWORDS = (
    "de",                  "a",
    "o",                   "que",
    "e",                   "do",
    "da",                  "em",
    "um",                  "para",
    "com",                 "n\xc3\xa3o",
    "uma",                 "os",
    "no",                  "se",
    "na",                  "por",
    "mais",                "as",
    "dos",                 "como",
    "mas",                 "ao",
    "ele",                 "das",
    "\xc3\xa0",            "seu",
    "sua",                 "ou",
    "quando",              "muito",
    "nos",                 "j\xc3\xa1",
    "eu",                  "tamb\xc3\xa9m",
    "s\xc3\xb3",           "pelo",
    "pela",                "at\xc3\xa9",
    "isso",                "ela",
    "entre",               "depois",
    "sem",                 "mesmo",
    "aos",                 "seus",
    "quem",                "nas",
    "me",                  "esse",
    "eles",                "voc\xc3\xaa",
    "essa",                "num",
    "nem",                 "suas",
    "meu",                 "\xc3\xa0s",
    "minha",               "numa",
    "pelos",               "elas",
    "qual",                "n\xc3\xb3s",
    "lhe",                 "deles",
    "essas",               "esses",
    "pelas",               "este",
    "dele",                "tu",
    "te",                  "voc\xc3\xaas",
    "vos",                 "lhes",
    "meus",                "minhas",
    "teu",                 "tua",
    "teus",                "tuas",
    "nosso",               "nossa",
    "nossos",              "nossas",
    "dela",                "delas",
    "esta",                "estes",
    "estas",               "aquele",
    "aquela",              "aqueles",
    "aquelas",             "isto",
    "aquilo",              "estou",
    "est\xc3\xa1",         "estamos",
    "est\xc3\xa3o",        "estive",
    "esteve",              "estivemos",
    "estiveram",           "estava",
    "est\xc3\xa1vamos",    "estavam",
    "estivera",            "estiv\xc3\xa9ramos",
    "esteja",              "estejamos",
    "estejam",             "estivesse",
    "estiv\xc3\xa9ssemos", "estivessem",
    "estiver",             "estivermos",
    "estiverem",           "hei",
    "h\xc3\xa1",           "havemos",
    "h\xc3\xa3o",          "houve",
    "houvemos",            "houveram",
    "houvera",             "houv\xc3\xa9ramos",
    "haja",                "hajamos",
    "hajam",               "houvesse",
    "houv\xc3\xa9ssemos",  "houvessem",
    "houver",              "houvermos",
    "houverem",            "houverei",
    "houver\xc3\xa1",      "houveremos",
    "houver\xc3\xa3o",     "houveria",
    "houver\xc3\xadamos",  "houveriam",
    "sou",                 "somos",
    "s\xc3\xa3o",          "era",
    "\xc3\xa9ramos",       "eram",
    "fui",                 "foi",
    "fomos",               "foram",
    "fora",                "f\xc3\xb4ramos",
    "seja",                "sejamos",
    "sejam",               "fosse",
    "f\xc3\xb4ssemos",     "fossem",
    "for",                 "formos",
    "forem",               "serei",
    "ser\xc3\xa1",         "seremos",
    "ser\xc3\xa3o",        "seria",
    "ser\xc3\xadamos",     "seriam",
    "tenho",               "tem",
    "temos",               "t\xc3\xa9m",
    "tinha",               "t\xc3\xadnhamos",
    "tinham",              "tive",
    "teve",                "tivemos",
    "tiveram",             "tivera",
    "tiv\xc3\xa9ramos",    "tenha",
    "tenhamos",            "tenham",
    "tivesse",             "tiv\xc3\xa9ssemos",
    "tivessem",            "tiver",
    "tivermos",            "tiverem",
    "terei",               "ter\xc3\xa1",
    "teremos",             "ter\xc3\xa3o",
    "teria",               "ter\xc3\xadamos",
    "teriam",
);

#mark utf8 entries as such
if ( $] > 5.007 ) {
    local $SIG{'__DIE__'}; #custom SIG causes trouble loading Encode
    require Encode;
    foreach my $w (@STOPWORDS) {
        $w = Encode::decode('utf8', $w);
    }
}

my %STOPWORDS;
@STOPWORDS{@STOPWORDS} = (1) x @STOPWORDS;
sub hashref { return {%STOPWORDS} }

1;
__END__

=head1 NAME

BigMed::Search::Stopwords::PT - Portuguese stopword support

=head1 DESCRIPTION

This module is used internally by BigMed::Search::Stopwords.

=head1 AUTHOR & COPYRIGHTS

This module and all Big Medium modules are copyright Josh Clark
and Global Moxie. All rights reserved.

Use of this module and the Big Medium content
management system are governed by Global Moxie's software licenses
and may not be used outside of the terms and conditions outlined
there.

For more information, visit the Global Moxie website at
L<http://globalmoxie.com/>.

Big Medium and Global Moxie are service marks of Global Moxie
and Josh Clark. All rights reserved.

=cut

